@@ -1,11 +1,8 @@
-use serde::{
-    Deserialize,
-    Serialize,
-};
+use serde::{Deserialize, Serialize};
 
 use crate::messaging::ExtensionMessages;
 
-/// Information about a extension
+/// Information about a extension instance
 #[derive(Serialize, Deserialize, Debug, Clone, PartialEq)]
 pub struct ExtensionInfo {
     pub id: String,
@@ -18,6 +15,10 @@ pub trait Extension {
     /// This will be called when the extension is loaded
     fn init(&mut self);
 
+    /// Unload method of the extension
+    /// This will be called when the extension is unloaded
+    fn unload(&mut self);
+
     /// Forward messages to the extension
     fn notify(&mut self, message: ExtensionMessages);
 
