@@ -1,27 +1,15 @@
 use crate::extensions::base::ExtensionInfo;
-use crate::extensions::manager::{
-    ExtensionsManager,
-    LoadedExtension,
-};
-use crate::filesystems::{
-    Filesystem,
-    LocalFilesystem,
-};
+use crate::extensions::manager::{ExtensionsManager, LoadedExtension};
+use crate::filesystems::{FileFormat, Filesystem, LocalFilesystem};
 use crate::messaging::ExtensionMessages;
-use crate::{
-    Errors,
-    ExtensionErrors,
-};
-use serde::{
-    Deserialize,
-    Serialize,
-};
+pub use crate::state_persistors::memory::MemoryPersistor;
+use crate::state_persistors::Persistor;
+use crate::{Errors, ExtensionErrors, LanguageServer, ManifestInfo};
+use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
 use std::fmt;
-use std::sync::{
-    Arc,
-    Mutex,
-};
+use std::sync::Arc;
+use tokio::sync::Mutex;
 
 #[derive(Clone)]
 pub enum TokenFlags {
@@ -67,45 +55,80 @@ impl StatesList {
         }
 
         self.states
-            .insert(state.id, Arc::new(Mutex::new(state.to_owned())));
+            .insert(state.data.id, Arc::new(Mutex::new(state.to_owned())));
 
         self
     }
 
     /// Notify all the extensions in a state about a message
-    pub fn notify_extensions(&self, message: ExtensionMessages) {
+    pub async fn notify_extensions(&self, message: ExtensionMessages) {
         let state_id = message.get_state_id();
         let state = self.states.get(&state_id);
         if let Some(state) = state {
-            let state = state.lock().unwrap();
+            let state = state.lock().await;
             state.notify_extensions(message);
         }
     }
 }
 
-/// a Tab state
+/// A Tab data
+#[derive(Serialize, Deserialize, Debug, Clone)]
+#[serde(tag = "tab_type")]
+pub enum TabData {
+    // Text Editor tab
+    TextEditor {
+        path: String,
+        filesystem: String,
+        format: FileFormat,
+        filename: String,
+        id: String,
+    },
+    // Basic tab (e.g. Settings)
+    Basic {
+        title: String,
+        id: String,
+    },
+}
+#[derive(Serialize, Deserialize, Clone, Debug, Default)]
+struct ViewPanel {
+    selected_tab_id: Option<String>,
+    tabs: Vec<TabData>,
+}
+
+type TabsViews = Vec<ViewPanel>;
+
+/// The data of a state
 #[derive(Serialize, Deserialize, Clone, Debug)]
-struct Tab {}
+pub struct StateData {
+    pub id: u8,
+    opened_tabs: Vec<TabsViews>,
+}
+
+impl Default for StateData {
+    fn default() -> Self {
+        Self {
+            id: 1,
+            opened_tabs: Vec::default(),
+        }
+    }
+}
 
 /// A state is like a small configuration, like a profile
-/// It stores what tabs do you have open, what extensions to load
-#[allow(dead_code)]
-#[derive(Serialize, Deserialize, Clone)]
+#[derive(Clone)]
 pub struct State {
-    #[serde(skip_serializing, skip_deserializing)]
     filesystems: HashMap<String, Arc<Mutex<Box<dyn Filesystem + Send>>>>,
-    #[serde(skip_serializing, skip_deserializing)]
     extensions_manager: ExtensionsManager,
-    opened_tabs: Vec<Tab>,
-    pub id: u8,
+    persistor: Option<Arc<Mutex<Box<dyn Persistor + Send>>>>,
+    pub data: StateData,
     tokens: Vec<String>,
+    language_servers: HashMap<String, LanguageServer>,
 }
 
 impl fmt::Debug for State {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("State")
-            .field("opened_tabs", &self.opened_tabs)
-            .field("id", &self.id)
+            .field("opened_tabs", &self.data.opened_tabs)
+            .field("id", &self.data.id)
             .finish()
     }
 }
@@ -113,6 +136,8 @@ impl fmt::Debug for State {
 impl Default for State {
     /// The default constructor will include:
     /// - LocalFilesystem
+    ///
+    /// But will not persist the state
     fn default() -> Self {
         let mut filesystems = HashMap::new();
 
@@ -121,20 +146,29 @@ impl Default for State {
         filesystems.insert("local".to_string(), Arc::new(Mutex::new(local_fs)));
 
         Self {
-            id: 1,
+            data: StateData::default(),
             filesystems,
             extensions_manager: ExtensionsManager::default(),
-            opened_tabs: Vec::new(),
             tokens: Vec::new(),
+            persistor: None,
+            language_servers: HashMap::new(),
         }
     }
 }
 
 impl State {
-    pub fn new(id: u8, extensions_manager: ExtensionsManager) -> Self {
+    pub fn new(
+        id: u8,
+        extensions_manager: ExtensionsManager,
+        mut persistor: Box<dyn Persistor + Send>,
+    ) -> Self {
+        // Retrieve opened tabs from the persistor
+        let state = persistor.load();
+
         State {
-            id,
+            data: StateData { id, ..state },
             extensions_manager,
+            persistor: Some(Arc::new(Mutex::new(persistor))),
             ..Default::default()
         }
     }
@@ -155,8 +189,9 @@ impl State {
     /// Run all the extensions in the manager
     pub async fn run_extensions(&self) {
         for ext in &self.extensions_manager.extensions {
-            if let LoadedExtension::FromExtension { plugin, .. } = ext {
+            if let LoadedExtension::ExtensionInstance { plugin, .. } = ext {
                 let mut ext_plugin = plugin.lock().await;
+                ext_plugin.unload();
                 ext_plugin.init();
             }
         }
@@ -165,10 +200,10 @@ impl State {
     /// Notify all the extensions in a state about a message, asynchronously and independently
     pub fn notify_extensions(&self, message: ExtensionMessages) {
         for ext in &self.extensions_manager.extensions {
-            if let LoadedExtension::FromExtension { plugin, .. } = ext {
+            if let LoadedExtension::ExtensionInstance { plugin, .. } = ext {
                 let ext_plugin = plugin.clone();
                 let message = message.clone();
-                tokio::task::spawn(async move {
+                tokio::spawn(async move {
                     let mut ext_plugin = ext_plugin.lock().await;
                     ext_plugin.notify(message.clone());
                 });
@@ -177,35 +212,43 @@ impl State {
     }
 
     /// Try to retrieve info about a perticular loaded extension
-    pub fn get_ext_info_by_id(&self, ext_id: &str) -> Result<ExtensionInfo, Errors> {
+    pub fn get_ext_info_by_id(&self, ext_id: &str) -> Result<ManifestInfo, Errors> {
         let extensions = &self.extensions_manager.extensions;
-        let result = extensions
-            .iter()
-            .find(|extension| {
-                if let LoadedExtension::FromFile { info, .. } = extension {
-                    info.id == ext_id
+        let result = extensions.iter().find_map(|extension| {
+            if let LoadedExtension::ManifestFile { manifest } = extension {
+                if manifest.info.extension.id == ext_id {
+                    Some(manifest.info.clone())
                 } else {
-                    false
+                    None
                 }
-            })
-            .map(|ext| ext.get_info());
+            } else if let LoadedExtension::ManifestBuiltin { info, .. } = extension {
+                if info.extension.id == ext_id {
+                    Some(info.clone())
+                } else {
+                    None
+                }
+            } else {
+                None
+            }
+        });
 
         result.ok_or(Errors::Ext(ExtensionErrors::ExtensionNotFound))
     }
 
-    /// Try to retrieve info about a perticular loaded extension's runtime
+    /// Try to retrieve info about a perticular loaded extension
     pub fn get_ext_run_info_by_id(&self, ext_id: &str) -> Result<ExtensionInfo, Errors> {
         let extensions = &self.extensions_manager.extensions;
-        let result = extensions
-            .iter()
-            .find(|extension| {
-                if let LoadedExtension::FromExtension { info, .. } = extension {
-                    info.id == ext_id
+        let result = extensions.iter().find_map(|extension| {
+            if let LoadedExtension::ExtensionInstance { info, .. } = extension {
+                if info.id == ext_id {
+                    Some(info.clone())
                 } else {
-                    false
+                    None
                 }
-            })
-            .map(|ext| ext.get_info());
+            } else {
+                None
+            }
+        });
 
         result.ok_or(Errors::Ext(ExtensionErrors::ExtensionNotFound))
     }
@@ -217,14 +260,41 @@ impl State {
         extensions
             .iter()
             .filter_map(|extension| {
-                if let LoadedExtension::FromFile { info, .. } = extension {
-                    Some(info.id.to_string())
+                if let LoadedExtension::ManifestBuiltin { info, .. } = extension {
+                    Some(info.extension.id.to_string())
+                } else if let LoadedExtension::ManifestFile { manifest } = extension {
+                    Some(manifest.info.extension.id.to_string())
                 } else {
                     None
                 }
             })
             .collect::<Vec<String>>()
     }
+
+    // Merge a new state data
+    pub async fn update(&mut self, new_data: StateData) {
+        self.data.opened_tabs = new_data.opened_tabs;
+
+        if let Some(persistor) = &self.persistor {
+            persistor.lock().await.save(&self.data);
+        }
+    }
+
+    // Register a new language server
+    pub async fn register_language_servers(
+        &mut self,
+        language_servers: HashMap<String, LanguageServer>,
+    ) {
+        self.language_servers.extend(language_servers);
+    }
+
+    // Register a new language server
+    pub async fn get_all_language_servers(&self) -> Vec<LanguageServer> {
+        self.language_servers
+            .values()
+            .cloned()
+            .collect::<Vec<LanguageServer>>()
+    }
 }
 
 // NOTE: It would be interesting to implement https://doc.rust-lang.org/std/ops/trait.AddAssign.html
@@ -233,12 +303,10 @@ impl State {
 #[cfg(test)]
 mod tests {
 
-    use crate::extensions::base::{
-        Extension,
-        ExtensionInfo,
-    };
+    use crate::extensions::base::{Extension, ExtensionInfo};
     use crate::extensions::manager::ExtensionsManager;
     use crate::messaging::ExtensionMessages;
+    use crate::state::MemoryPersistor;
 
     use super::State;
 
@@ -261,6 +329,10 @@ mod tests {
                 todo!()
             }
 
+            fn unload(&mut self) {
+                todo!()
+            }
+
             fn notify(&mut self, _message: ExtensionMessages) {
                 todo!()
             }
@@ -273,7 +345,7 @@ mod tests {
     fn get_info() {
         let mut manager = ExtensionsManager::default();
         manager.register("sample", get_sample_extension());
-        let test_state = State::new(0, manager);
+        let test_state = State::new(0, manager, Box::new(MemoryPersistor::new()));
 
         let ext_info = test_state.get_ext_run_info_by_id("sample");
         assert!(ext_info.is_ok());
